@@ -17,10 +17,8 @@
 
 #define COMMENT_PREFIX "\"#"
 
-static void
-cb_window_icon(girara_session_t* session, const char* UNUSED(name),
-    girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data))
-{
+static void cb_window_icon(girara_session_t* session, const char* UNUSED(name), girara_setting_type_t UNUSED(type),
+                           const void* value, void* UNUSED(data)) {
   g_return_if_fail(session != NULL && value != NULL);
 
   if (session->gtk.window == NULL) {
@@ -30,24 +28,20 @@ cb_window_icon(girara_session_t* session, const char* UNUSED(name),
   girara_set_window_icon(session, value);
 }
 
-static void
-cb_font(girara_session_t* session, const char* UNUSED(name),
-    girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data))
-{
+static void cb_font(girara_session_t* session, const char* UNUSED(name), girara_setting_type_t UNUSED(type),
+                    const void* value, void* UNUSED(data)) {
   g_return_if_fail(session != NULL && value != NULL);
 
   css_template_fill_font(session->private_data->csstemplate, value);
 }
 
-static void
-cb_color(girara_session_t* session, const char* name,
-    girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data))
-{
+static void cb_color(girara_session_t* session, const char* name, girara_setting_type_t UNUSED(type), const void* value,
+                     void* UNUSED(data)) {
   g_return_if_fail(session != NULL && value != NULL);
 
   const char* str_value = value;
 
-  GdkRGBA color = { 0, 0, 0, 0 };
+  GdkRGBA color = {0, 0, 0, 0};
   gdk_rgba_parse(&color, str_value);
 
   char* colorstr = gdk_rgba_to_string(&color);
@@ -55,17 +49,13 @@ cb_color(girara_session_t* session, const char* name,
   g_free(colorstr);
 }
 
-static void
-cb_guioptions(girara_session_t* session, const char* UNUSED(name),
-    girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data))
-{
+static void cb_guioptions(girara_session_t* session, const char* UNUSED(name), girara_setting_type_t UNUSED(type),
+                          const void* value, void* UNUSED(data)) {
   g_return_if_fail(session != NULL && value != NULL);
 
   /* set default values */
   bool show_commandline = false;
   bool show_statusbar   = false;
-  bool show_hscrollbar  = false;
-  bool show_vscrollbar  = false;
 
   /* evaluate input */
   const char* input         = value;
@@ -73,20 +63,14 @@ cb_guioptions(girara_session_t* session, const char* UNUSED(name),
 
   for (size_t i = 0; i < input_length; i++) {
     switch (input[i]) {
-      /* command line */
-      case 'c':
-        show_commandline = true;
-        break;
-      /* statusbar */
-      case 's':
-        show_statusbar = true;
-        break;
-      case 'h':
-        show_hscrollbar = true;
-        break;
-      case 'v':
-        show_vscrollbar = true;
-        break;
+    /* command line */
+    case 'c':
+      show_commandline = true;
+      break;
+    /* statusbar */
+    case 's':
+      show_statusbar = true;
+      break;
     }
   }
 
@@ -106,29 +90,25 @@ cb_guioptions(girara_session_t* session, const char* UNUSED(name),
     session->global.hide_statusbar = true;
     gtk_widget_hide(session->gtk.statusbar);
   }
-
-  scrolled_window_set_scrollbar_visibility(
-    GTK_SCROLLED_WINDOW(session->gtk.view), show_hscrollbar, show_vscrollbar);
 }
 
-void
-girara_config_load_default(girara_session_t* session)
-{
+void girara_config_load_default(girara_session_t* session) {
   if (session == NULL) {
     return;
   }
 
   /* values */
-  const int statusbar_h_padding   = 8;
-  const int statusbar_v_padding   = 2;
-  const int window_width          = 800;
-  const int window_height         = 600;
-  const int n_completion_items    = 15;
-  girara_mode_t normal_mode = session->modes.normal;
+  const int statusbar_h_padding = 8;
+  const int statusbar_v_padding = 2;
+  const int window_width        = 800;
+  const int window_height       = 600;
+  const int n_completion_items  = 15;
+  girara_mode_t normal_mode     = session->modes.normal;
 
   /* other values */
   session->global.autohide_inputbar = true;
 
+  /* clang-format off */
   /* settings */
   girara_setting_add(session, "font",                     "monospace normal 9", STRING,  FALSE, _("Font"), cb_font, NULL);
   girara_setting_add(session, "default-fg",               "#DDDDDD",            STRING,  FALSE,  _("Default foreground color"), cb_color, NULL);
@@ -149,8 +129,6 @@ girara_config_load_default(girara_session_t* session)
   girara_setting_add(session, "notification-warning-bg",  "#F3F000",            STRING,  FALSE,  _("Warning notifaction background color"), cb_color, NULL);
   girara_setting_add(session, "notification-fg",          "#000000",            STRING,  FALSE,  _("Notification foreground color"), cb_color, NULL);
   girara_setting_add(session, "notification-bg",          "#FFFFFF",            STRING,  FALSE,  _("Notification background color"), cb_color, NULL);
-  girara_setting_add(session, "scrollbar-fg",             "#DDDDDD",            STRING,  FALSE,  _("Scrollbar foreground color"), cb_color, NULL);
-  girara_setting_add(session, "scrollbar-bg",             "#000000",            STRING,  FALSE,  _("Scrollbar background color"), cb_color, NULL);
   girara_setting_add(session, "word-separator",           " /.-=&#?",           STRING,  TRUE,  NULL, NULL, NULL);
   girara_setting_add(session, "window-width",             &window_width,        INT,     TRUE,  _("Initial window width"), NULL, NULL);
   girara_setting_add(session, "window-height",            &window_height,       INT,     TRUE,  _("Initial window height"), NULL, NULL);
@@ -217,16 +195,15 @@ girara_config_load_default(girara_session_t* session)
   girara_shortcut_mapping_add(session, "set",              girara_sc_set);
   girara_shortcut_mapping_add(session, "toggle_inputbar",  girara_sc_toggle_inputbar);
   girara_shortcut_mapping_add(session, "toggle_statusbar", girara_sc_toggle_statusbar);
+  /* clang-format on */
 }
 
-bool
-girara_config_handle_add(girara_session_t* session, const char* identifier, girara_command_function_t handle)
-{
+bool girara_config_handle_add(girara_session_t* session, const char* identifier, girara_command_function_t handle) {
   g_return_val_if_fail(session != NULL, false);
   g_return_val_if_fail(identifier != NULL, false);
 
   girara_session_private_t* session_private = session->private_data;
-  bool found = false;
+  bool found                                = false;
 
   /* search for existing config handle */
   for (size_t idx = 0; idx != girara_list_size(session_private->config.handles); ++idx) {
@@ -250,9 +227,7 @@ girara_config_handle_add(girara_session_t* session, const char* identifier, gira
   return true;
 }
 
-void
-girara_config_handle_free(girara_config_handle_t* handle)
-{
+void girara_config_handle_free(girara_config_handle_t* handle) {
   if (handle == NULL) {
     return;
   }
@@ -261,37 +236,46 @@ girara_config_handle_free(girara_config_handle_t* handle)
   g_free(handle);
 }
 
-static bool
-config_parse(girara_session_t* session, const char* path)
-{
+static bool config_parse(girara_session_t* session, const char* path) {
   /* open file */
-  FILE* file = girara_file_open(path, "r");
+  g_autoptr(GFile) f = g_file_new_for_path(path);
+  if (f == NULL) {
+    girara_debug("failed to open config file '%s'", path);
+    return false;
+  }
 
-  if (file == NULL) {
+  g_autoptr(GFileInputStream) stream = g_file_read(f, NULL, NULL);
+  if (stream == NULL) {
+    girara_debug("failed to open config file '%s'", path);
+    return false;
+  }
+
+  g_autoptr(GDataInputStream) datastream = g_data_input_stream_new(G_INPUT_STREAM(stream));
+  if (datastream == NULL) {
     girara_debug("failed to open config file '%s'", path);
     return false;
   }
 
   /* read lines */
-  char* line = NULL;
+  char* line               = NULL;
+  gsize line_length        = 0;
   unsigned int line_number = 1;
-  while ((line = girara_file_read_line(file)) != NULL) {
+  while ((line = g_data_input_stream_read_line(datastream, &line_length, NULL, NULL)) != NULL) {
     /* skip empty lines and comments */
-    if (strlen(line) == 0 || strchr(COMMENT_PREFIX, line[0]) != NULL) {
+    if (line_length == 0 || strchr(COMMENT_PREFIX, line[0]) != NULL) {
       g_free(line);
       continue;
     }
 
-    girara_list_t* argument_list = girara_list_new2(g_free);
+    g_autoptr(girara_list_t) argument_list = girara_list_new_with_free(g_free);
     if (argument_list == NULL) {
       g_free(line);
-      fclose(file);
       return false;
     }
 
-    gchar** argv = NULL;
-    gint    argc = 0;
-    GError* error = NULL;
+    gchar** argv            = NULL;
+    gint argc               = 0;
+    g_autoptr(GError) error = NULL;
 
     /* parse current line */
     if (g_shell_parse_argv(line, &argc, &argv, &error) != FALSE) {
@@ -300,16 +284,11 @@ config_parse(girara_session_t* session, const char* path)
         girara_list_append(argument_list, argument);
       }
     } else {
-      girara_list_free(argument_list);
       if (error->code != G_SHELL_ERROR_EMPTY_STRING) {
         girara_error("Could not parse line %d in '%s': %s", line_number, path, error->message);
-
-        g_error_free(error);
-        fclose(file);
         g_free(line);
         return false;
       } else {
-        g_error_free(error);
         g_free(line);
         continue;
       }
@@ -320,15 +299,13 @@ config_parse(girara_session_t* session, const char* path)
       if (argc != 2) {
         girara_warning("Could not process line %d in '%s': usage: include path.", line_number, path);
       } else {
-        char* newpath = NULL;
+        g_autofree char* newpath = NULL;
         if (g_path_is_absolute(argv[1]) == TRUE) {
           newpath = g_strdup(argv[1]);
         } else {
-          char* basename = g_path_get_dirname(path);
-          char* tmp = g_build_filename(basename, argv[1], NULL);
-          newpath = girara_fix_path(tmp);
-          g_free(tmp);
-          g_free(basename);
+          g_autofree char* basename = g_path_get_dirname(path);
+          g_autofree char* tmp      = g_build_filename(basename, argv[1], NULL);
+          newpath                   = girara_fix_path(tmp);
         }
 
         if (g_strcmp0(newpath, path) == 0) {
@@ -339,7 +316,6 @@ config_parse(girara_session_t* session, const char* path)
             girara_warning("Could not process line %d in '%s': failed to load '%s'.", line_number, path, newpath);
           }
         }
-        g_free(newpath);
       }
     } else {
       /* search for config handle */
@@ -360,18 +336,14 @@ config_parse(girara_session_t* session, const char* path)
     }
 
     line_number++;
-    girara_list_free(argument_list);
     g_strfreev(argv);
     g_free(line);
   }
 
-  fclose(file);
   return true;
 }
 
-void
-girara_config_parse(girara_session_t* session, const char* path)
-{
+void girara_config_parse(girara_session_t* session, const char* path) {
   girara_debug("reading configuration file '%s'", path);
   config_parse(session, path);
 }
